function entropyFocusingTest()
%
% entropyFocusingTest.m
%
% This is a demo of the Entropy-Based camera focus measure calculated using
% discrete cosine transform of the focusing window. The demostration loads
% an image and evaluates the measure within a predefined focusing window at
% different gaussian kernels. Large values of the measure correspond to
% sharper images.
% 
% run: entropyFocusingTest()
%
%
% Author: Matej Kristan, 2007
% For more details, please see this paper:
%   M. Kristan, J. Per, M. Pere, S. Kovai. 
%   "A Bayes-Spectral-Entropy-Based Measure of Camera Focus Using 
%   a Discrete Cosine Transform". Pattern Recognition Letters, 
%   27(13): 1419-1580, October 2006.

% parameters used in the paper
low_f = 0 ;                    % bottom threshold on frequency
high_f = 6 ;                   % top threshold on frequency
window = [ 8, 8 ] ;            % subwindow size

% parameters for the demo
cx = 220 ; cy = 120 ;          % location of the focusing window
delta_w = 40 ;                 % size of the focusing window
filename = 'Strawberry.bmp' ;  % input image
S = [0 : 1 : 10] ;             % Gaussian kernel sizes used in demo
noise = 0.0 ;                  % noise in terms of intensity range, e.g., noise = 0.09



% read image and convert to grayscale
n = imread(filename) ;
n = rgb2gray(n) ;
n = double(n) ;

% compute bounds of the focusing window
x0 = cx - delta_w ;
y0 = cy - delta_w;
x1 = cx + delta_w + 1;
y1 = cy + delta_w + 1;
w = x1 - x0 + 1; 
h = y1 - y0 + 1; 
    
figure(1); clf ;
sub_cols = ceil(sqrt(length(S))) ;
sub_rows = ceil(length(S)/sub_cols) ;


Be = [] ;
for i = 1 : length(S)
    s = S(i) ; % select Gaussian kernel size

    % blur image
    n0 = blurGaussian( s, n ) ;

   % add uniform noise
   n0 = addNoise(n0, noise) ;

   % extract focusing image
   subImage = n0( y0 : y1 , x0 : x1 ) ;

   % display image and focusing window
   subplot(sub_cols,sub_rows,i);
   imagesc(uint8(n0)) ; colormap gray ; hold on ;
   rectangle( 'Position', [ x0, y0, w, h ], 'LineWidth', 2, 'EdgeColor', 'r' ) ;   
   axis equal; axis tight; drawnow ; 
   
   % evaluate focus measure 
   measure = BayesDCT( subImage, window, low_f, high_f ) ;

   % display value of the measure
   msg = sprintf( 'M^8_{Be} = %1.5f', measure ) ;
   title(msg) ; drawnow ;
   
   Be = [Be, measure] ;
end

% display dependence between the measure and image focus
figure(2) ; clf; hold on;
plot(S,Be,'b-'); plot(S,Be,'co'); title('Focus measure w.r.t. Gaussian bluring')
xlabel('\sigma') ; ylabel('Value of the measure M^8_{Be}')

function n2 = addNoise(n, NoiseAdd)

dom = (max(max(n))-min(min(n))) ;
n2 = n + randn(size(n))*(dom * NoiseAdd) ;

function n2 = blurGaussian( s, n )

d = 6*s + 1 ;
if s <= 0 s = 1 ; end
h = fspecial( 'gaussian', d, s ) ;
n2 = filter2( h, n, 'same' ) ;